"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const quidproquo_core_1 = require("quidproquo-core");
const quidproquo_webserver_1 = require("quidproquo-webserver");
const awsLambdaUtils_1 = require("../../../awsLambdaUtils");
const getProcessTransformEventParams = (appName) => {
    return ({ eventParams: [apiGatewayEvent, context] }) => __awaiter(void 0, void 0, void 0, function* () {
        const path = (apiGatewayEvent.path || '').replace(new RegExp(`^(\/${appName})/`), '/');
        console.log('getProcessTransformEventParams', JSON.stringify(apiGatewayEvent));
        return (0, quidproquo_core_1.actionResult)({
            path,
            query: Object.assign(Object.assign({}, (apiGatewayEvent.multiValueQueryStringParameters || {})), (apiGatewayEvent.queryStringParameters || {})),
            body: apiGatewayEvent.body ? JSON.parse(apiGatewayEvent.body) : undefined,
            headers: apiGatewayEvent.headers,
            method: apiGatewayEvent.httpMethod,
            correlation: context.awsRequestId,
            sourceIp: apiGatewayEvent.requestContext.identity.sourceIp,
        });
    });
};
const getProcessTransformResponseResult = (configs) => {
    // We might need to JSON.stringify the body.
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        // Validate response
        // if !valid actionResultError
        console.log('getProcessTransformResponseResult', JSON.stringify(payload));
        return (0, quidproquo_core_1.actionResult)({
            statusCode: payload.response.result.statusCode,
            body: payload.response.result.body,
            headers: Object.assign(Object.assign({ 'Content-Type': 'application/json' }, quidproquo_webserver_1.qpqWebServerUtils.getCorsHeaders(configs, {}, payload.transformedEventParams.headers)), (payload.response.headers || {})),
        });
    });
};
const getProcessAutoRespond = (configs) => {
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        if (payload.transformedEventParams.method === 'OPTIONS') {
            return (0, quidproquo_core_1.actionResult)({
                result: {
                    statusCode: 200,
                    headers: quidproquo_webserver_1.qpqWebServerUtils.getCorsHeaders(configs, {}, payload.transformedEventParams.headers),
                },
            });
        }
        return (0, quidproquo_core_1.actionResult)(null);
    });
};
const getProcessMatchStory = (routes) => {
    return (payload) => __awaiter(void 0, void 0, void 0, function* () {
        // Sort the routes by string length
        // Note: We may need to filter variable routes out {} as the variables are length independent
        const sortedRoutes = routes
            .filter((r) => r.method === payload.transformedEventParams.method)
            .sort((a, b) => {
            if (a.path.length < b.path.length)
                return -1;
            if (a.path.length > b.path.length)
                return 1;
            return 0;
        });
        // Find the most relevant match
        const matchedRoute = sortedRoutes
            .map((r) => ({
            match: (0, awsLambdaUtils_1.matchUrl)(r.path, payload.transformedEventParams.path),
            route: r,
        }))
            .find((m) => m.match.didMatch);
        if (!matchedRoute) {
            return (0, quidproquo_core_1.actionResultError)(quidproquo_core_1.ErrorTypeEnum.NotFound, 'route not found');
        }
        return (0, quidproquo_core_1.actionResult)({
            src: matchedRoute.route.src,
            runtime: matchedRoute.route.runtime,
            options: matchedRoute.match.params || {},
        });
    });
};
exports.default = (config) => {
    const routes = quidproquo_webserver_1.qpqWebServerUtils.getAllRoutes(config);
    const appName = quidproquo_core_1.qpqCoreUtils.getAppName(config);
    return {
        [quidproquo_core_1.EventActionType.TransformEventParams]: getProcessTransformEventParams(appName),
        [quidproquo_core_1.EventActionType.TransformResponseResult]: getProcessTransformResponseResult(config),
        [quidproquo_core_1.EventActionType.AutoRespond]: getProcessAutoRespond(config),
        [quidproquo_core_1.EventActionType.MatchStory]: getProcessMatchStory(routes),
    };
};
