"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.listFiles = void 0;
const client_s3_1 = require("@aws-sdk/client-s3");
const quidproquo_core_1 = require("quidproquo-core");
const s3Client_1 = __importDefault(require("./s3Client"));
const listFiles = (drive, folder = '', maxKeys = 1000, pageToken) => __awaiter(void 0, void 0, void 0, function* () {
    const validatedPrefix = `${folder}${folder.endsWith(quidproquo_core_1.filePathDelimiter) || !folder ? '' : quidproquo_core_1.filePathDelimiter}`;
    const bucketParams = {
        Bucket: drive,
        Delimiter: quidproquo_core_1.filePathDelimiter,
        Prefix: validatedPrefix,
        ContinuationToken: pageToken,
        MaxKeys: maxKeys,
    };
    // Declare truncated as a flag that the while loop is based on.
    let files = [];
    const response = yield s3Client_1.default.send(new client_s3_1.ListObjectsV2Command(bucketParams));
    if (response.CommonPrefixes && !bucketParams.ContinuationToken) {
        files = [
            ...files,
            ...response.CommonPrefixes.filter((cp) => !!cp.Prefix).map((cp) => ({
                filepath: cp.Prefix,
                drive: drive,
                isDir: true,
            })),
        ];
    }
    files = [
        ...files,
        ...(response.Contents || [])
            .filter((c) => !!c.Key && c.Key != folder)
            .map((item) => ({
            filepath: item.Key,
            isDir: item.Key.endsWith(quidproquo_core_1.filePathDelimiter),
            hashMd5: item.ETag,
        })),
    ];
    return {
        fileInfos: files,
        pageToken: response.NextContinuationToken,
    };
});
exports.listFiles = listFiles;
